@@ -66,7 +66,6 @@ struct _MimeEditDialogPrivate
 	gboolean      component_active         : 1;
 	gboolean      default_action_active    : 1;
 	gboolean      custom_action            : 1;
-	gboolean      use_cat_dfl              : 1;
 };
 
 static guint dialog_signals[LAST_SIGNAL];
@@ -456,6 +455,8 @@ safe_set_entry (MimeEditDialog *dialog, char const *widget, char const *txt)
 static void
 fill_dialog (MimeEditDialog *dialog)
 {
+	gchar *category_name;
+	
 	g_return_if_fail (dialog->p->info != NULL);
 
 	mime_type_info_load_all (dialog->p->info);
@@ -464,10 +465,10 @@ fill_dialog (MimeEditDialog *dialog)
 		dialog->p->info->description);
 	safe_set_entry (dialog, "mime_type_entry",
 		dialog->p->info->mime_type);
-	safe_set_entry (dialog, "category_entry",
-		mime_type_info_get_category_name (dialog->p->info));
-
-	dialog->p->use_cat_dfl = dialog->p->info->use_category;
+	category_name = mime_type_info_get_category_name (dialog->p->info);
+	safe_set_entry (dialog, "category_entry", category_name);
+	g_free (category_name);
+	
 	update_sensitivity (dialog);
 
 	gnome_icon_entry_set_filename (GNOME_ICON_ENTRY (WID ("icon_entry")), mime_type_info_get_icon_path (dialog->p->info));
@@ -567,10 +568,10 @@ populate_application_list (MimeEditDialog *dialog)
 
 	menu = GTK_MENU (gtk_menu_new ());
 
-	app_list = gnome_vfs_application_registry_get_applications (dialog->p->info->mime_type);
+	app_list  = gnome_vfs_mime_get_short_list_applications (dialog->p->info->mime_type);
 
 	for (tmp = app_list, i = 0; tmp != NULL; tmp = tmp->next, i++) {
-		app = gnome_vfs_application_registry_get_mime_application (tmp->data);
+		app = tmp->data;
 
 		if (dialog->p->info->default_action != NULL &&
 		    dialog->p->info->default_action->id != NULL &&
@@ -990,8 +991,8 @@ response_cb (MimeEditDialog *dialog, gint response_id)
 static void
 update_sensitivity (MimeEditDialog *dialog) 
 {
-	gtk_widget_set_sensitive (WID ("component_box"), dialog->p->component_active && !dialog->p->use_cat_dfl);
-	gtk_widget_set_sensitive (WID ("default_action_box"), dialog->p->default_action_active && !dialog->p->use_cat_dfl);
-	gtk_widget_set_sensitive (WID ("program_entry_box"), dialog->p->custom_action && !dialog->p->use_cat_dfl);
-	gtk_widget_set_sensitive (WID ("needs_terminal_toggle"), dialog->p->custom_action && !dialog->p->use_cat_dfl);
+	gtk_widget_set_sensitive (WID ("component_box"), dialog->p->component_active);
+	gtk_widget_set_sensitive (WID ("default_action_box"), dialog->p->default_action_active);
+	gtk_widget_set_sensitive (WID ("program_entry_box"), dialog->p->custom_action);
+	gtk_widget_set_sensitive (WID ("needs_terminal_toggle"), dialog->p->custom_action);
 }
