@@ -15,7 +15,6 @@
 #include <X11/Xft/Xft.h>
 #endif /* HAVE_XFT2 */
 
-#include "theme-common.h"
 #include "capplet-util.h"
 #include "activate-settings-daemon.h"
 #include "gconf-property-editor.h"
@@ -27,7 +26,6 @@ static void cb_show_details (GtkWidget *button,
 
 #define GTK_FONT_KEY           "/desktop/gnome/interface/font_name"
 #define DESKTOP_FONT_KEY       "/apps/nautilus/preferences/desktop_font"
-#define WINDOW_TITLE_FONT_KEY  "/desktop/gnome/applications/window_manager/titlebar_font"
 
 #define METACITY_DIR "/apps/metacity/general"
 #define WINDOW_TITLE_FONT_KEY METACITY_DIR "/titlebar_font"
@@ -164,6 +162,8 @@ open_pattern (FcPattern   *pattern,
 	int screen = gdk_x11_get_default_screen ();
 	
 	res_pattern = XftFontMatch (xdisplay, screen, pattern, &result);
+	if (res_pattern == NULL)
+		return NULL;
 	
 	FcPatternDel (res_pattern, FC_HINTING);
 	FcPatternAddBool (res_pattern, FC_HINTING, hinting != HINT_NONE);
@@ -306,13 +306,17 @@ font_render_get_gconf (Antialiasing *antialiasing,
 	int val;
 
 	val = ANTIALIAS_GRAYSCALE;
-	if (antialias_str)
+	if (antialias_str) {
 		gconf_string_to_enum (antialias_enums, antialias_str, &val);
+		g_free (antialias_str);
+	}
 	*antialiasing = val;
 
 	val = HINT_FULL;
-	if (hint_str)
+	if (hint_str) {
 		gconf_string_to_enum (hint_enums, hint_str, &val);
+		g_free (hint_str);
+	}
 	*hinting = val;
 
 	g_object_unref (client);
@@ -435,8 +439,9 @@ setup_dialog (GladeXML *dialog)
   GObject *peditor;
 
   client = gconf_client_get_default ();
-
+	  
   gconf_client_add_dir (client, "/desktop/gnome/interface", GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
+  gconf_client_add_dir (client, "/apps/nautilus/preferences", GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
   gconf_client_add_dir (client, METACITY_DIR, GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
 #ifdef HAVE_XFT2  
   gconf_client_add_dir (client, FONT_RENDER_DIR, GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
@@ -530,6 +535,8 @@ enum_group_load (EnumGroup *group)
 	if (str)
 		gconf_string_to_enum (group->enums, str, &val);
 
+	g_free (str);
+	
 	in_change = TRUE;
 	
 	for (tmp_list = group->items; tmp_list; tmp_list = tmp_list->next) {
@@ -669,7 +676,10 @@ cb_details_response (GtkDialog *dialog, gint response_id)
 		capplet_help (GTK_WINDOW (dialog),
 			      "wgoscustdesk.xml",
 			      "goscustdesk-38");
-	else
+	else if (response_id == 1) {
+		/* "Go to font folder" was clicked */
+		g_spawn_command_line_async ("nautilus --no-desktop fonts:///", NULL);
+	} else
 		gtk_widget_hide (GTK_WIDGET (dialog));
 }
 
@@ -683,8 +693,16 @@ cb_show_details (GtkWidget *button,
 		GConfClient *client = gconf_client_get_default ();
 		GladeXML *dialog = glade_xml_new (GLADEDIR "/font-properties.glade", "render_details", NULL);
 		GtkWidget *dpi_spinner;
+		GnomeVFSURI *uri;
 
 		details_dialog = WID ("render_details");
+		uri = gnome_vfs_uri_new ("fonts:///");
+		if (uri == NULL) {
+			gtk_widget_hide (WID ("go_to_font_button"));
+		} else {
+			gnome_vfs_uri_unref (uri);
+			gtk_widget_show (WID ("go_to_font_button"));
+		}
 
 		gtk_window_set_transient_for (GTK_WINDOW (details_dialog), parent);
 
