@@ -121,6 +121,7 @@ create_image (ThemeThumbnailData *theme_thumbnail_data,
   GdkPixbuf *folder_icon;
   char *folder_icon_name;
   char *foo;
+
   settings = gtk_settings_get_default ();
   g_object_set (G_OBJECT (settings),
 		"gtk-theme-name", (char *) theme_thumbnail_data->control_theme_name->data,
@@ -197,8 +198,17 @@ create_image (ThemeThumbnailData *theme_thumbnail_data,
 
   /* Handle the icon theme */
   icon_theme = gnome_icon_theme_new ();
+  gnome_icon_theme_set_allow_svg (icon_theme, TRUE);
   gnome_icon_theme_set_custom_theme (icon_theme, (char *) theme_thumbnail_data->icon_theme_name->data);
+
+  /* Have to try both "folder" and "gnome-fs-directory" seems themes seem to use either name */
   folder_icon_name = gnome_icon_theme_lookup_icon (icon_theme, "folder", 48, NULL, NULL);
+  if (folder_icon_name == NULL) {
+    folder_icon_name = gnome_icon_theme_lookup_icon (icon_theme, "gnome-fs-directory", 48, NULL, NULL);
+  }
+ 
+  g_object_unref (icon_theme);
+ 
   if (folder_icon_name != NULL)
     {
       folder_icon = gdk_pixbuf_new_from_file (folder_icon_name, NULL);
@@ -341,6 +351,7 @@ message_from_capplet (GIOChannel   *source,
 	    {
 	      write (pipe_from_factory_fd[1], pixels + (rowstride)*i, ICON_SIZE_WIDTH * gdk_pixbuf_get_n_channels (pixbuf));
 	    }
+	  g_object_unref (pixbuf);
 	  theme_thumbnail_data->status = READY_FOR_THEME;
 	  g_byte_array_set_size (theme_thumbnail_data->control_theme_name, 0);
 	  g_byte_array_set_size (theme_thumbnail_data->wm_theme_name, 0);
@@ -378,7 +389,6 @@ message_from_child (GIOChannel   *source,
                                     1024,
                                     &bytes_read,
                                     NULL);
-
   switch (status)
     {
     case G_IO_STATUS_NORMAL:
@@ -397,7 +407,7 @@ message_from_child (GIOChannel   *source,
 	    memcpy (pixels + rowstride * i, async_data.data->data + 4 * ICON_SIZE_WIDTH * i, ICON_SIZE_WIDTH * 4);
 
 	  scaled_pixbuf = gdk_pixbuf_scale_simple (pixbuf, ICON_SIZE_WIDTH/2, ICON_SIZE_HEIGHT/2, GDK_INTERP_BILINEAR);
-	  g_hash_table_insert (theme_hash, async_data.meta_theme_name, scaled_pixbuf);
+	  g_hash_table_insert (theme_hash, g_strdup(async_data.meta_theme_name), scaled_pixbuf);
 	  g_object_unref (pixbuf);
 
  	  (* async_data.func) (scaled_pixbuf, async_data.user_data);
@@ -432,8 +442,18 @@ message_from_child (GIOChannel   *source,
   return TRUE;
 }
 
+void  
+theme_thumbnail_invalidate_cache (GnomeThemeMetaInfo *meta_theme_info)
+{
+  gboolean success;
+
+  success = g_hash_table_remove (theme_hash, meta_theme_info->name);
+  printf ("Success is %d\n", success);
+}
+
 GdkPixbuf *
-generate_theme_thumbnail (GnomeThemeMetaInfo *meta_theme_info)
+generate_theme_thumbnail (GnomeThemeMetaInfo *meta_theme_info,
+			  gboolean            clear_cache)
 {
   GdkPixbuf *retval = NULL;
   GdkPixbuf *pixbuf = NULL;
@@ -446,7 +466,10 @@ generate_theme_thumbnail (GnomeThemeMetaInfo *meta_theme_info)
   pixbuf = g_hash_table_lookup (theme_hash, meta_theme_info->name);
   if (pixbuf != NULL)
     {
-      return pixbuf;
+      if (clear_cache)
+	g_hash_table_remove (theme_hash, meta_theme_info->name);
+      else
+	return pixbuf;
     }
 
   pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8, ICON_SIZE_WIDTH, ICON_SIZE_HEIGHT);
@@ -463,12 +486,22 @@ generate_theme_thumbnail (GnomeThemeMetaInfo *meta_theme_info)
 
   for (i = 0; i < ICON_SIZE_HEIGHT; i++)
     {
-      read (pipe_from_factory_fd[0], pixels + (rowstride)*i, ICON_SIZE_WIDTH * gdk_pixbuf_get_n_channels (pixbuf));
+      gint j = 0;
+      gint bytes_read;
+
+      do
+	{
+	  bytes_read = read (pipe_from_factory_fd[0], pixels + (rowstride)*i + j, ICON_SIZE_WIDTH * gdk_pixbuf_get_n_channels (pixbuf) - j);
+	  if (bytes_read > 0)
+	    j += bytes_read;
+	}
+      while (j < ICON_SIZE_WIDTH * gdk_pixbuf_get_n_channels (pixbuf));
     }
 
   retval = gdk_pixbuf_scale_simple (pixbuf, ICON_SIZE_WIDTH/2, ICON_SIZE_HEIGHT/2, GDK_INTERP_BILINEAR);
-
-  g_hash_table_insert (theme_hash, meta_theme_info->name, retval);
+  g_object_unref (pixbuf);
+  
+  g_hash_table_insert (theme_hash, g_strdup (meta_theme_info->name), retval);
   return retval;
 }
 
@@ -560,5 +593,5 @@ theme_thumbnail_factory_init (int argc, char *argv[])
   async_data.meta_theme_name = NULL;
   async_data.data = g_byte_array_new ();
 
-  theme_hash = g_hash_table_new_full (g_str_hash, g_str_equal, NULL, g_object_unref);
+  theme_hash = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_object_unref);
 }
