@@ -25,7 +25,9 @@
 #define __CAPPLET_DIR_H
 
 #include <gnome.h>
+#include <libgnome/gnome-desktop-item.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
+#include <libgnomevfs/gnome-vfs.h>
 
 #define CAPPLET_DIR_ENTRY(obj) ((CappletDirEntry *) obj)
 #define CAPPLET_DIR(obj) ((CappletDir *) obj)
@@ -48,10 +50,11 @@ typedef enum {
 struct _CappletDirEntry 
 {
 	CappletEntryType type;
-	GnomeDesktopEntry *entry;
+	GnomeDesktopItem *entry;
+	gchar **exec;
 	gchar *label;
 	gchar *icon;
-	gchar *path;
+        GnomeVFSURI *uri;
 	GdkPixbuf *pb;
 	CappletDir *dir;	
 };
